package com.boldchat.integrations;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;

import org.apache.commons.codec.binary.Base64;

public class RestRequest {
	private final RequestBase requestBase;
	private String path;
	private String origin;
	private StringBuilder parameters = new StringBuilder();

	public RestRequest(RequestBase requestBase, String path, String origin) {
		this.path = path;
		this.requestBase = requestBase;
		this.origin = origin;
	}

	public RestRequest(RequestBase requestBase, String path) {
		this.path = path;
		this.requestBase = requestBase;
	}

	public RestRequest(RequestBase requestBase) {
		this.path = "";
		this.requestBase = requestBase;
	}

	public String getPathAndParameters() {
		if(parameters.length() > 0) {
			String paramSeparator = path.contains("?") ? "&": "?";
			return path + paramSeparator + parameters.toString();
		} else {
			return path;
		}
	}

	public void addParameter(String name, String value) {
		if(parameters.length() > 0) {
			parameters.append("&");
		}
		parameters.append(urlEncode(name));
		if(value != null && !value.isEmpty()) {
			parameters.append("=");
			parameters.append(urlEncode(value));
		}
	}

	public String fetchRawResult() throws IOException {
		String result = "";
			URL url = new URL(requestBase.getUrl() + getPathAndParameters());
			URLConnection urlConnection = url.openConnection();
			addAuthorizationHeader(urlConnection);
			if(this.origin != null) {
				urlConnection.setRequestProperty("Origin", this.origin);
			}
			InputStream is = urlConnection.getInputStream();
			InputStreamReader isr = new InputStreamReader(is);

			int numCharsRead;
			char[] charArray = new char[1024];
			StringBuilder sb = new StringBuilder();
			while ((numCharsRead = isr.read(charArray)) > 0) {
				sb.append(charArray, 0, numCharsRead);
			}
			result = sb.toString();

		return result;
	}

	protected void addAuthorizationHeader(URLConnection urlConnection) {
		if(requestBase.getUserName() != null && requestBase.getPassword() != null) {
            String authString = requestBase.getUserName() + ":" + requestBase.getPassword();
			urlConnection.setRequestProperty("Authorization", getAuthorizationHeader(authString));
		}
	}

	public static String getAuthorizationHeader(String authString) {
		byte[] authEncBytes = Base64.encodeBase64(authString.getBytes());
		return "Basic " + getString(authEncBytes);
	}

	protected String getBaseUrl() {
		return requestBase.getUrl();
	}

	private static String getString(byte[] bytes) {
		try {
			return new String(bytes, "UTF-8");
		} catch(UnsupportedEncodingException e) {
			throw new AssertionError(e);
		}
	}

	private String urlEncode(String value) {
		try {
			return URLEncoder.encode(value, "UTF-8");
		} catch(UnsupportedEncodingException e) {
			throw new AssertionError(e);
		}
	}
}
