package com.boldchat.integrations.sample;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Map;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.boldchat.integrations.ApiSecurityFilter;
import com.boldchat.integrations.ApiSettingsFactory;
import com.boldchat.integrations.AuthToken;
import com.boldchat.integrations.ServletUtil;
import com.boldchat.integrations.StringUtil;
import com.boldchat.integrations.sample.models.ReturnContext;
import com.google.gson.Gson;

public class SampleServlet extends HttpServlet {

    private Gson gson = new Gson();
    private ApiSettingsFactory settingsFactory;

    @Override
    public void init(ServletConfig servletConfig) throws ServletException {
        String settingsFactoryName = servletConfig.getInitParameter("ApiSettingsFactory");
        settingsFactory = newSettingsFactory(settingsFactoryName);
    }

    protected static ApiSettingsFactory newSettingsFactory(String settingsFactoryName) throws ServletException {
        if(settingsFactoryName == null || settingsFactoryName.isEmpty()) {
            throw new ServletException("ApiSettingsFactory is a required parameter to ApiSecurityFilter");
        }
        try {
            Class factoryClass = Class.forName(settingsFactoryName);
            return (ApiSettingsFactory)factoryClass.newInstance();
        } catch (ClassNotFoundException |InstantiationException | IllegalAccessException e) {
            throw new ServletException(settingsFactoryName);
        }
    }

	@Override
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		String result = "";
        SampleService service = new SampleService(settingsFactory);
		try {
			String servletPath = request.getServletPath();
            if (servletPath.endsWith("/getTicket")) {
                result = service.getTicket(request.getParameter("ticketID"));
            } else if (servletPath.endsWith("/searchTickets")) {
                result = service.searchTickets(request.getParameter("ticketID"), request.getParameter("name"), request.getParameter("email"));
			} else if (servletPath.endsWith("/linkTicket")) {
                result = service.linkTicket(request.getParameterMap(), getAuthToken(request));
            } else if (servletPath.endsWith("/services") || servletPath.endsWith("/index.html")) {
                StringBuilder newPath = new StringBuilder("/sample");
                Map params = request.getParameterMap();
                int count = 0;
                for(String param : new String[] {"ChatID", "SampleTicketNumber", "ChatName", "VisitEmailAddress", "auth"}) {
                    String value = StringUtil.getAsString(params.get(param));
                    if(value != null) {
                        String sign = (count == 0) ? "?" : "&";
                        newPath.append(sign).append(ServletUtil.encode(param)).append('=').append(ServletUtil.encode(value.trim()));
                        count ++;
                    }
                }
                response.setStatus(HttpServletResponse.SC_SEE_OTHER);
                response.setHeader("Location", newPath.toString());
            }
		} catch (Exception e) {
            ReturnContext<String> returnContext = new ReturnContext(false, e.getMessage());
            writeResult(gson.toJson(returnContext), response.getOutputStream());
            return;
		}
        writeResult(result, response.getOutputStream());
	}

	@Override
	protected void doPost(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
		doGet(req, res);
	}

    protected void writeResult(String result, OutputStream outputStream) throws IOException {
        try {
            outputStream.write(result.getBytes("UTF-8"));
            outputStream.flush();
        } finally {
            if(outputStream != null) {
                outputStream.close();
            }
        }
    }

    private AuthToken getAuthToken(HttpServletRequest request) {
        String cookie = ServletUtil.getCookieValue(request, ApiSecurityFilter.AUTH_COOKIE_NAME);
        String authParam = (cookie != null) ? cookie : request.getParameter("auth");
        return new AuthToken(authParam);
    }
}
