// the BoldChat client software passes us an UpdateChatUrl which we use to save the ticket number to
// the chat record.  these parameters are added to the UpdateChatUrl.
var linkerUrlFields = 'auth={a}&ChatID={c}&CustomFields.' + ticketSystem.customField + '={t}';

angular.module('TicketIntegrationFrontend', ['ngRoute', 'ngSanitize'])

.config(['$routeProvider', function($routeProvider) {

        $routeProvider
            .when('/search', {
                controller: 'SearchCtrl',
                templateUrl: 'search.html'
            })
            .when('/view', {
                controller: "ViewCtrl",
                templateUrl: 'view-page.html'
            })
            .when('/results', {
                controller: 'ResultsCtrl',
                templateUrl: 'results.html'
            })
            .otherwise({
                redirectTo: '/search'
            });

    }])

.run(['$rootScope', 'UrlParams', function($rootScope, UrlParams) {

        // set the page title
        $rootScope.title = ticketSystem.title;

        // check ticket and redirect if present
        var TicketID = UrlParams.get(ticketSystem.customField);
        if (TicketID && TicketID.length > 0 && UrlParams.path() != '/check') {
            UrlParams.path('/view');
        }

    }])

.service('UrlParams', ['$location', function($location) {

        var params = {};

        return {
            get: function(name) {
                var savedParam = params[name];
                if (savedParam != undefined) {
                    return savedParam;
                } else if (find = (new RegExp('[?&]' + encodeURIComponent(name) + '=([^&]*)')).exec(location.search)) {
                    return decodeURIComponent(find[1]);
                } else {
                    return $location.search()[name];
                }
            },
            set: function(name, value) {
                params[name] = value;
            },
            path: function(newPath) {
                return $location.path(newPath);
            },
            getDomainPath: function() {
                return $location.absUrl().replace(/\/[?#].*/, '');
            }

        };

    }])

.factory('Ticket', ['$http', 'UrlParams', '$rootScope', function($http, UrlParams, $rootScope) {

        var searchResults = null;

        function callHttp(method, url) {
            return $http[method](url);
        }

        function callWrappedHttp(method, callback, url) {

            callHttp(method, url)
                .success(callbackWrapper(callback, true))
                .error(callbackWrapper(callback, false));

            function callbackWrapper(callback, success) {
                return function (data, status, headers, config) {
                    if (success) {
                        callback(data.Status == "success", data.Message);
                    } else {
                        callback(false, data);
                    }
                }
            }

        }

        function addParams(url, TicketID) {
            return url + linkerUrlFields.replace('{c}', encodeURIComponent(UrlParams.get('ChatID')))
                .replace('{a}', encodeURIComponent(UrlParams.get('auth')))
                .replace('{t}', encodeURIComponent(TicketID));
        }

        return {

            search: function(ticket, name, email, callback) {
                var search = ticket ? "ticketID=" + encodeURIComponent(ticket) + "&" : "";
                if (name) search += "name=" + encodeURIComponent(name) + "&";
                if (email) search += "email=" + encodeURIComponent(email);
                var callbackIntercept = function(success, data) {
                    searchResults = (success && data !== "null") ? data : null;
                    callback(success, data);
                };
                callWrappedHttp("get", callbackIntercept, ticketSystem.searchTicketsUrl + search);
            },

            getTicket: function(TicketID, callback) {
                callWrappedHttp("get", callback, ticketSystem.getTicketUrl + encodeURIComponent(TicketID));
            },

            link: function(TicketID, callback) {
                var linkUrl = addParams(ticketSystem.linkTicketUrl, TicketID);
                callWrappedHttp("get", callback, linkUrl);
            },

            unlink: function(TicketID, callback) {
                var linkUrl = addParams(ticketSystem.linkTicketUrl, ""); //Calling linkTicket with a blank TicketID will unlink ticket
                callWrappedHttp("get", callback, linkUrl);
            },

            getSearchResults: function() {
                return searchResults;
            }

        };

    }])

.controller('SearchCtrl', ['$scope', 'Ticket', 'UrlParams', function($scope, Ticket, UrlParams) {

        $scope.name = UrlParams.get('ChatName');
        $scope.email = UrlParams.get('VisitEmailAddress');

        $scope.search = function() {
            if (!checkInput()) return;
            $scope.isSearching = true;
            var TicketID = $scope.ticket;
            if ($scope.name) UrlParams.set('ChatName', $scope.name);
            if ($scope.email) UrlParams.set('VisitEmailAddress', $scope.email);
            Ticket.search(TicketID, $scope.name, $scope.email, searchCallback);
        };

        function checkInput() {
            if (fieldIsEmpty('name') && fieldIsEmpty('email') && fieldIsEmpty('ticket')) {
                $scope.errorText = "Enter search criteria";
                $scope.searchFailed = true;
                return false;
            } else {
                $scope.searchFailed = false;
                return true;
            }
        }

        function fieldIsEmpty(field) {
            return (!$scope[field] || $scope[field].length == 0);
        }

        function searchCallback(success, data) {
            $scope.isSearching = false;
            if (success) {
                UrlParams.path('/results')
            } else {
                $scope.errorText = "No results found";
                $scope.searchFailed = true;
            }
        }

    }])

.controller('ViewCtrl', ['$scope', 'Ticket', 'UrlParams', function ($scope, Ticket, UrlParams) {

        var TicketID = UrlParams.get(ticketSystem.customField);

        if (!TicketID) {

            $scope.notFound = true;

        } else {

            $scope.isLoading = true;
            Ticket.getTicket(TicketID, function (success, ticketData) {

                $scope.isLoading = false;

                if (!success) {
                    $scope.notFound = true;

                } else {
                    $scope.selectedTicket = ticketData;
                    $scope.linked = true;
                }

            });

        }

    }])

.controller('ResultsCtrl', ['$scope', 'Ticket', 'UrlParams', function ($scope, Ticket, UrlParams) {

        $scope.results = Ticket.getSearchResults();
        $scope.system = ticketSystem.name;

        $scope.goToSearch = function() {
            UrlParams.path('/search');
        };

        $scope.toggleTicket = function(ticket) {
            if ($scope.selectedTicket == ticket) {
                $scope.selectedTicket = null;
            } else {
                $scope.selectedTicket = ticket;
            }
        };

        $scope.navigate = function(event, index) {

            var max = $scope.results.length - 1;

            if (event.keyCode == 38) { // up arrow key
                $scope.focusIndex = (!index) ? max : index - 1;
                event.preventDefault();

            } else if (event.keyCode == 40) { // down arrow key
                $scope.focusIndex = (index === max || index === undefined || index === null) ? 0 : index + 1;
                event.preventDefault();

            } else if (event.keyCode == 32 || event.keyCode == 13) {    // enter or space keys
                $scope.toggleTicket(index);
                event.preventDefault();
            }

        }

    }])

.controller('HeaderCtrl', ['$scope', 'UrlParams', function($scope, UrlParams) {

        var setSearchPageBoolean = function() {
            $scope.isSearchPage = UrlParams.path() == '/search' || UrlParams.path() == '/create';
        };
        setSearchPageBoolean();

        $scope.$on('$locationChangeSuccess', setSearchPageBoolean);

        $scope.goToSearch = function() {
            UrlParams.path('/search');
        };

        $scope.header = ticketSystem.title;

    }])

.directive('ticketView', [function() {
        return {
            restrict: 'A',
            templateUrl: 'ticket-view.html',
            scope: {
                ticket: '=',
                readOnly: '@'
            },
            controller: ['$scope', 'UrlParams', 'Ticket', '$window', '$filter', function($scope, UrlParams, Ticket, $window, $filter) {

                $scope.$watch('ticket', function(newVal, oldVal) {
                    if (newVal) {
                        $scope.isError = false;
                        $scope.linked = (newVal.TicketID == UrlParams.get(ticketSystem.customField));
                    }
                });

                $scope.link = function() {
                    if (UrlParams.get(ticketSystem.customField)) {
                        if (!confirm("Only one ticket can be linked to a chat at a time." +
                            "\nThis action will unlink the previous ticket." +
                            "\nPrevious Ticket: " + UrlParams.get(ticketSystem.customField) +
                            "\nNew Ticket: " + $scope.ticket.TicketID +
                            "\nContinue?")) {
                            return;
                        }
                    }
                    callLinker('link');
                };

                $scope.unlink = function() {
                    callLinker('unlink');
                };


                function callLinker(linkFunction) {
                    var path = (linkFunction === 'link') ? '/view' : '/search';
                    var TicketID = (linkFunction === 'link') ? $scope.ticket.TicketID : '';
                    $scope.isLoading = true;
                    $scope.isError = false;
                    Ticket[linkFunction]($scope.ticket.TicketID, function(success, data) {
                        $scope.isLoading = false;
                        if (success) {
                            UrlParams.set(ticketSystem.customField, TicketID);
                            UrlParams.path(path);
                        } else {
                            $scope.isError = true;
                            $scope.errorText = 'An error occurred while ' + linkFunction + 'ing the ticket';
                        }
                    });
                }

            }]
        }
    }])

.directive('loadingSpinner', [function() {
        return {
            restrict: 'A',
            templateUrl: 'loading.html',
            scope: {
                text: '@text'
            }
        }
    }])

.directive('focus', [function() {
        return {
            restrict: 'A',
            scope: {
                focus: '@'
            },
            link: function(scope, element, attrs) {
                scope.$watch('focus', function (newVal, oldVal) {
                    if (newVal && newVal === "true") {
                        element[0].focus();
                    }
                });
            }
        }
    }])

.filter('displayCase', function() {
    return function(input) {
        return input[0].toUpperCase() + input.slice(1).replace(/[A-Z]+/g, ' $&');
    }
});

